package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.SignerWithRecovery;
import cn.com.infosec.crypto.digests.RIPEMD128Digest;
import cn.com.infosec.crypto.digests.RIPEMD160Digest;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.params.ParametersWithSalt;
import cn.com.infosec.crypto.params.RSAKeyParameters;
import java.security.SecureRandom;

public class ISO9796d2PSSSigner implements SignerWithRecovery {
   public static final int TRAILER_IMPLICIT = 188;
   public static final int TRAILER_RIPEMD160 = 12748;
   public static final int TRAILER_RIPEMD128 = 13004;
   public static final int TRAILER_SHA1 = 13260;
   private Digest digest;
   private AsymmetricBlockCipher cipher;
   private SecureRandom random;
   private byte[] standardSalt;
   private int hLen;
   private int trailer;
   private int keyBits;
   private byte[] block;
   private byte[] mBuf;
   private int messageLength;
   private int saltLength;
   private boolean fullMessage;
   private byte[] recoveredMessage;

   public ISO9796d2PSSSigner(AsymmetricBlockCipher cipher, Digest digest, int saltLength, boolean implicit) {
      this.cipher = cipher;
      this.digest = digest;
      this.hLen = digest.getDigestSize();
      this.saltLength = saltLength;
      if (implicit) {
         this.trailer = 188;
      } else if (digest instanceof SHA1Digest) {
         this.trailer = 13260;
      } else if (digest instanceof RIPEMD160Digest) {
         this.trailer = 12748;
      } else {
         if (!(digest instanceof RIPEMD128Digest)) {
            throw new IllegalArgumentException("no valid trailer for digest");
         }

         this.trailer = 13004;
      }

   }

   public ISO9796d2PSSSigner(AsymmetricBlockCipher cipher, Digest digest, int saltLength) {
      this(cipher, digest, saltLength, false);
   }

   private boolean isSameAs(byte[] a, byte[] b) {
      if (this.messageLength != b.length) {
         return false;
      } else {
         for(int var3 = 0; var3 != b.length; ++var3) {
            if (a[var3] != b[var3]) {
               return false;
            }
         }

         return true;
      }
   }

   private void clearBlock(byte[] block) {
      for(int var2 = 0; var2 != block.length; ++var2) {
         block[var2] = 0;
      }

   }

   private void ItoOSP(int i, byte[] sp) {
      sp[0] = (byte)(i >>> 24);
      sp[1] = (byte)(i >>> 16);
      sp[2] = (byte)(i >>> 8);
      sp[3] = (byte)(i >>> 0);
   }

   private void LtoOSP(long l, byte[] sp) {
      sp[0] = (byte)((int)(l >>> 56));
      sp[1] = (byte)((int)(l >>> 48));
      sp[2] = (byte)((int)(l >>> 40));
      sp[3] = (byte)((int)(l >>> 32));
      sp[4] = (byte)((int)(l >>> 24));
      sp[5] = (byte)((int)(l >>> 16));
      sp[6] = (byte)((int)(l >>> 8));
      sp[7] = (byte)((int)(l >>> 0));
   }

   private byte[] maskGeneratorFunction1(byte[] Z, int zOff, int zLen, int length) {
      byte[] var5 = new byte[length];
      byte[] var6 = new byte[this.hLen];
      byte var7 = 4;
      byte[] var8 = new byte[4];
      int var9 = 0;
      this.digest.reset();

      int var10;
      while(var9 < length / (var10 = this.hLen)) {
         this.ItoOSP(var9, var8);
         this.digest.update(Z, zOff, zLen);
         this.digest.update(var8, 0, var7);
         this.digest.doFinal(var6, 0);
         int var11 = var9 * (var10 = this.hLen);
         System.arraycopy(var6, 0, var5, var11, var10);
         ++var9;
      }

      if (var9 * var10 < length) {
         this.ItoOSP(var9, var8);
         this.digest.update(Z, zOff, zLen);
         this.digest.update(var8, 0, var7);
         this.digest.doFinal(var6, 0);
         int var12;
         int Z1 = var9 * (var12 = this.hLen);
         var12 = length - var9 * var12;
         System.arraycopy(var6, 0, var5, Z1, var12);
      }

      return var5;
   }

   public void init(boolean forSigning, CipherParameters param) {
      int var3 = this.saltLength;
      RSAKeyParameters var4;
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom param1;
         var4 = (RSAKeyParameters)(param1 = (ParametersWithRandom)param).getParameters();
         if (forSigning) {
            this.random = param1.getRandom();
         }
      } else if (param instanceof ParametersWithSalt) {
         ParametersWithSalt var10000 = (ParametersWithSalt)param;
         var4 = (RSAKeyParameters)((ParametersWithSalt)param).getParameters();
         byte[] var8 = this.standardSalt = var10000.getSalt();
         var3 = var8.length;
         if (var8.length != this.saltLength) {
            throw new IllegalArgumentException("Fixed salt is of wrong length");
         }
      } else {
         var4 = (RSAKeyParameters)param;
         if (forSigning) {
            SecureRandom param2;
            param2 = new SecureRandom.<init>();
            this.random = param2;
         }
      }

      this.cipher.init(forSigning, var4);
      byte[] forSigning1;
      this.block = forSigning1 = new byte[((this.keyBits = var4.getModulus().bitLength()) + 7) / 8];
      if (this.trailer == 188) {
         this.mBuf = new byte[forSigning1.length - this.digest.getDigestSize() - var3 - 1 - 1];
      } else {
         this.mBuf = new byte[forSigning1.length - this.digest.getDigestSize() - var3 - 1 - 2];
      }

      this.reset();
   }

   public void update(byte b) {
      int var2;
      byte[] var3;
      if ((var2 = this.messageLength) < (var3 = this.mBuf).length) {
         this.messageLength = var2 + 1;
         var3[var2] = b;
      } else {
         this.digest.update(b);
      }

   }

   public void update(byte[] in, int off, int len) {
      while(len > 0 && this.messageLength < this.mBuf.length) {
         this.update(in[off]);
         ++off;
         --len;
      }

      if (len > 0) {
         this.digest.update(in, off, len);
      }

   }

   public void reset() {
      this.digest.reset();
      this.messageLength = 0;
      byte[] var1;
      if ((var1 = this.mBuf) != null) {
         this.clearBlock(var1);
      }

      if ((var1 = this.recoveredMessage) != null) {
         this.clearBlock(var1);
         this.recoveredMessage = null;
      }

      this.fullMessage = false;
   }

   public byte[] generateSignature() {
      int var1;
      byte[] var2 = new byte[var1 = this.digest.getDigestSize()];
      this.digest.doFinal(var2, 0);
      byte[] var3 = new byte[8];
      this.LtoOSP((long)(this.messageLength * 8), var3);
      this.digest.update(var3, 0, 8);
      var3 = this.mBuf;
      int var4 = this.messageLength;
      this.digest.update(var3, 0, var4);
      this.digest.update(var2, 0, var1);
      byte[] var8;
      if ((var8 = this.standardSalt) == null) {
         var8 = new byte[this.saltLength];
         this.random.nextBytes(var8);
      }

      this.digest.update(var8, 0, var8.length);
      int var9;
      var3 = new byte[var9 = this.digest.getDigestSize()];
      this.digest.doFinal(var3, 0);
      byte var11 = 2;
      if (this.trailer == 188) {
         var11 = 1;
      }

      byte[] var10003 = var8;
      byte[] var10004 = var8;
      byte[] var5;
      byte[] var10006 = var5 = this.block;
      int var6;
      var10006[var1 = var10006.length - (var6 = this.messageLength) - var8.length - this.hLen - var11 - 1] = 1;
      byte[] var7 = this.mBuf;
      int var10005 = ++var1;
      System.arraycopy(var7, 0, var5, var1, var6);
      var8 = this.block;
      int var12 = var10005 + this.messageLength;
      var6 = var10004.length;
      System.arraycopy(var10003, 0, var8, var12, var6);
      var1 = this.block.length - this.hLen - var11;
      var8 = this.maskGeneratorFunction1(var3, 0, var9, var1);

      byte[] var10000;
      for(var9 = 0; var9 != var8.length; ++var9) {
         var10000 = this.block;
         var10000[var9] ^= var8[var9];
      }

      int var10 = (var8 = this.block).length - (var9 = this.hLen) - var11;
      System.arraycopy(var3, 0, var8, var10, var9);
      if ((var1 = this.trailer) == 188) {
         var10000 = this.block;
         var10000[var10000.length - 1] = -68;
      } else {
         var10000 = var2 = this.block;
         int var13 = var1;
         int var14 = var1;
         var1 = var2.length - 2;
         var2[var1] = (byte)(var14 >>> 8);
         var1 = var2.length - 1;
         var10000[var1] = (byte)var13;
      }

      byte[] var10001 = var8 = this.block;
      var10001[0] = (byte)(var10001[0] & 127);
      var10000 = this.cipher.processBlock(var8, 0, var8.length);
      this.clearBlock(this.mBuf);
      this.clearBlock(this.block);
      this.messageLength = 0;
      return var10000;
   }

   public boolean verifySignature(byte[] param1) {
      // $FF: Couldn't be decompiled
   }

   public boolean hasFullMessage() {
      return this.fullMessage;
   }

   public byte[] getRecoveredMessage() {
      return this.recoveredMessage;
   }
}
